/*
 * $XConsortium: XedwListP.h,v 1.12 89/12/11 15:09:04 kit Exp $
 *
 * Copyright 1989 Massachusetts Institute of Technology
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Chris D. Peterson, MIT X Consortium
 */


/* 
 * XedwListP.h - Private definitions for XedwList widget
 * 
 * This is the XedwList widget, it is useful to display a xedwList, without the
 * overhead of having a widget for each item in the xedwList.  It allows 
 * the user to select an item in a xedwList and notifies the application through
 * a callback function.
 *
 *	Created: 	8/13/88
 *	By:		Chris D. Peterson
 *                      MIT - Project Athena
 */

#ifndef _XedwListP_h
#define _XedwListP_h

/***********************************************************************
 *
 * XedwList Widget Private Data
 *
 ***********************************************************************/

#include <X11/Xaw/SimpleP.h>
#include "XedwList.h"

#define NO_HIGHLIGHT            XDTM_LIST_NONE
#define OUT_OF_RANGE            -1
#define OKAY                     0
#define XedwOff			 0
#define XedwOn			 1

/* New fields for the XedwList widget class record */

typedef struct {int foo;} XedwListClassPart;

/* Full class record declaration */
typedef struct _XedwListClassRec {
    CoreClassPart      	core_class;
    SimpleClassPart    	simple_class;
    XedwListClassPart	xedwList_class;
} XedwListClassRec;

extern XedwListClassRec xedwListClassRec;

typedef struct _LinkedList {
  int index;
  struct _LinkedList *next;
} LinkedList;

/* New fields for the XedwList widget record */
typedef struct {
  /* resources */
  Pixel         foreground;
  Dimension     internal_width,
                internal_height,
                column_space,
                row_space,
                icon_width,
                icon_height;
  int           default_cols;
  Boolean       force_cols,
                paste,
                vertical_cols,
                show_icons,	/* Show icons with list */
  		multiple;       /* Allow multiple selections */
  int           longest;
  int           nitems;		/* number of items in the xedwList. */
  XFontStruct	*font;
  XedwList  **xedwList;
  XtCallbackList  callback;

  /* private state */

  LinkedList  *is_highlighted;	/* set to the items currently highlighted. */
  int         highlight,	/* set to the item that should be highlighted.*/
              col_width,	/* width of each column. */
              row_height,	/* height of each row. */
              nrows,		/* number of rows in the xedwList. */
              ncols;		/* number of columns in the xedwList. */
  GC	      normgc,		/* a couple o' GC's. */
              revgc,
              graygc;		/* used when inactive. */
  Pixmap      default_icon;

} XedwListPart;


/****************************************************************
 *
 * Full instance record declaration
 *
 ****************************************************************/

typedef struct _XedwListRec {
  CorePart	core;
  SimplePart	simple;
  XedwListPart	xedwList;
} XedwListRec;

#endif /* _XedwListP_h */



