/*****************************************************************************/
/** File    : actions.c                                                     **/
/** Purpose : main callbacks                                                **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/
#include "xproc.h"

#include <sys/proc.h>

#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
   
#include "interface.h"
#include "structure.h"

extern XedwList ** Modiflist_icon (Widget,String,
				   Pixmap,Pixmap,int); /* modif_icon.c */
extern XedwList ** Create_host_ps_list (String); /* list.c */
extern window_type * Create_Host_Interface (String); /* list.c */
extern window_type * Goto_window_host(String); /* goto.c */
extern void Confirm (String, String, 
		     void *, void *); /* confirm.c */
extern void Problem_w (String); /* problem.c */
extern void Quitcancel (Widget, XtPointer, 
			XtPointer ); /* confirm.c */
extern void Update_Highlighted_ps_List (); /* list.c */
extern void Unhighlight_all (Widget); /* list.c */
extern struct host_ps * Goto_host (String); /* goto.c */
extern struct icon_name * Goto_struct (dqueue, String); /* goto.c */
extern struct ps * Extract_pid(String,String); /* state.c */
extern int Execute(int,String,String,dqueue);
extern int Give_ps (int,String,struct host_ps *);
extern int Nb_ps_by_host (String); /* list.c */
extern int Nb_struct (dqueue); /* list.c */
extern void End_connect(int);
extern int Ask_daemon_and_build_list(String,Widget); /* list.c */

extern int sock;
extern Pixmap grey_pixmap;

void Ok_Choice (Widget, XtPointer, XtPointer);
void Exec_Com (Widget , XtPointer , XtPointer );
void Exec_Choice_Com (Widget , XtPointer , XtPointer );
void Update (Widget ,XtPointer ,XtPointer);

void Action_received (Widget w,XtPointer client_data,XtPointer call_data)
{
  /* call_backs from main interface 
     client_data is Command or Host */
  
  XedwListReturnStruct *list;
  window_type * w_host;
  struct host_ps * tmp_host_ps;
  /* list contains hightlighted item of Widget w */

  list = XedwListShowCurrent (w);
  
  switch ((main_choice)client_data){
  case Host :
    /* mouse button pressed in Widget Host */
    if (list->xedwList_index != -1) 
      {
	XedwListUnhighlight (w,list->xedwList_index);     
	
	w_host = Goto_window_host (list->string);
	/* the host name is list->string */
	
	if (w_host==NULL) /* Widget not exist */
	  w_host = Create_Host_Interface (list->string);
	else
	  if(Ask_daemon_and_build_list(list->string,
					  w_host->list.itself)==-1)
	    return;
	if (w_host!=NULL)
	  { /* mask host name icon */
	    XedwListChange (w,
			    Modiflist_icon (w,
					    list->string,
					    0,
					    grey_pixmap,
					    GXand),
			    Nb_struct (config_list[Host]),
			    0,
			    True);
	    XtPopup (w_host->toplevel,XtGrabNone); /* display host widget */
	  }
      }
    break;
  case Command:
    /* mouse button pressed in Widget Command */
    if (list->xedwList_index != -1) /* Is not empty list ? */
      { 
	Update_Highlighted_ps_List ();
	    Confirm ("Execute\n",
		     (String)list->string, 
		     ((Goto_struct(config_list[Command],
				   (String)list->string)->parameters
		       )==0)?Exec_Choice_Com:Exec_Com, 
		     Quitcancel);
	XedwListUnhighlight (w,list->xedwList_index);     
      }
    break;
  }
}

void Close_Host (Widget w , XButtonEvent* button)
{
  /* actions raise by mouse button on Widget Host */
  window_type * w_host;
  XedwListReturnStruct *list;
  
  list = XedwListShowCurrent (w);

  if (list->xedwList_index != -1) /* Is not empty list ? */
    {
      XedwListUnhighlight (w,list->xedwList_index);
      /* host name is list->string */
      w_host=Goto_window_host (list->string);
      
      if (w_host!=NULL) 
	{ /* unmask host name icon */
	  XedwListChange (w,
			  Modiflist_icon (w, 
					   list->string,
					   StringtoPixmap 
					  ((Goto_struct (config_list[Host],
							 (String)list->string)
					    )->icon),
					   0,
					   GXcopy),
			  Nb_struct (config_list[Host]),
			  0,
			  True);
	  Unhighlight_all (w_host->list.itself);
	  XtPopdown (w_host->toplevel); /* unmap host processes Widget */ 
	}
    }
}

void Exec_Com (Widget widget, XtPointer command, XtPointer unused)
{
  /* build list of highlighted processes per host
     and give to daemon lists and command */

  Arg arglist[1];
  String host_name;
  XedwListReturnStruct * tmp;
  dqueue construct_ps; 
  struct ps * tmp_ps;
  int i;
  
  construct_ps=dq_create();
  dq_set_destroy(construct_ps,(void *)XtFree);
  
  for (i=0;i<nb_w_ps;i++) /* go over all host processes Widget displayed */
    {
      XtSetArg (arglist[0], XtNtitle,&host_name);
      XtGetValues (w_ps[i].toplevel, arglist, 1);

      /* build list of processes */
      dq_go_beginning(construct_ps);
      tmp = w_ps[i].list.highlighted_ps;
      if( tmp!=NULL && (tmp->xedwList_index!=-1))
	{
	  while (tmp!=NULL && tmp->xedwList_index!=-1)
	    {
	      tmp_ps=(struct ps *)Our_XtMalloc(sizeof(struct ps));
	      tmp_ps->pid=(Extract_pid(host_name,tmp->string))->pid;
	      dq_insert(construct_ps,tmp_ps);
	      tmp = tmp->next;
	    }
	  dq_go_beginning(construct_ps);

	  /* give list construct_ps to daemon */
	  if(Execute(sock,
		     host_name,
		     (Goto_struct (config_list[Command],
				   (String)command))->function,
		     construct_ps)==-1)
	    Problem_w(host_name);
	  dq_clear(construct_ps);
	}
    }
  Update (widget,NULL,NULL);
  XtPopdown (w_confirm.itself);/* close confirm Widget */
}

void Exec_Choice_Com (Widget widget, XtPointer command,XtPointer unused)
{
  /* params=0 in .xprocrc so  no need of 
     highlighted processes per host (empty_queue). 
     user has choice an host(w_confirm.choice.chosen_name)
     and a command */
 
  if(w_confirm.choice.chosen_name!=NULL) /* host name chosen ? */
    {
      dq_go_beginning(empty_queue);
      if(Execute(sock,
		 w_confirm.choice.chosen_name,
		 (Goto_struct (config_list[Command],
			       (String)command))->function,
		 empty_queue)==-1)
	Problem_w(w_confirm.choice.chosen_name);
      XtFree(w_confirm.choice.chosen_name);
    }
  Update (widget,NULL,NULL);
  XtPopdown (w_confirm.itself);/* close confirm Widget */
}

void Ok_Choice (Widget w, XtPointer client_data,XtPointer call_data)
{
  /* callback raise when selecting host name in confirm Widget */
 
 XawListReturnStruct *item = (XawListReturnStruct*)call_data;
  XtFree(w_confirm.choice.chosen_name);
  w_confirm.choice.chosen_name=XtNewString(item->string);
}

void Update (Widget widget, XtPointer client_data, XtPointer call_data)
{
  Arg arglist[1];
  String name;
  int i;

  for(i=0;i<nb_w_ps;i++)
    {
      XtSetArg (arglist[0], XtNtitle,&name);
      XtGetValues (w_ps[i].toplevel, arglist, 1);
      (void)Ask_daemon_and_build_list(name,w_ps[i].list.itself);
    }
  Update_Highlighted_ps_List ();
}
