/*****************************************************************************/
/** File    : daemon.c                                                      **/
/** Purpose : Body of daemon                                                **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/
/* Copyright 1992 DESS Genie Logiciel */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/param.h>
#include <sys/user.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <fcntl.h>
#include "structure.h"
#include "defines.h"
#include "dqueue.h"


extern void *Our_alloc(long);                       /* From our_alloc.c */
extern struct host_ps *Init_structure_host(int);    /* From init_host_root.c */
extern int Create_sock_UNIX(struct sockaddr *,      /* From manip_sock.c */
			    char *);
extern int Create_sock_INET(struct sockaddr_in *,
			    int port);              /* From manip_sock.c */
extern int Read_nb(int,
		   char *,
		   struct sockaddr_in *);           /* From manip_sock.c */
extern void Write_nb(int,
		     char *,
		     struct sockaddr_in *);         /* From manip_sock.c */
extern void Send_message(int,
			 char *,
			 struct sockaddr_in *);     /* From manip_sock.c */
extern int Recv_message(int,
			char *,
			struct sockaddr_in *);      /* From manip_sock.c */
extern void Send_struct_host(void (*)(),
			     int,
			     struct sockaddr_in *,
			     struct host_ps *);     /* From manip_host.c */
extern int Receive_struct_host(int (*)(),
			       int,
			       struct sockaddr_in *,
			       struct host_ps *);   /* From manip_host.c */
extern dqueue Receive_order(int,
			    char *,
			    char *);                /* From execute.c */
extern void Exec_command(char *,dqueue);            /* From give_command.c */

extern void free(void *);



void Sigchld()
{
  /* Sigchld is used to avoid defunct processes. */

  while(wait3(NULL,WNOHANG,NULL)>0);
}


main()
{
   struct sockaddr ad_sock_interface;
   struct sockaddr_in ad_sock_daemons;
   struct hostent *host_ptr;
   struct ps *cell_ps;
   struct host_ps *structure_host_from_daemon,*structure_on_the_other_host;
   struct host_ps *structure_remote_host;
   char *host_name,*message,*uid_string,*command,*pid_string,*this_host;
   char *from_host_name,*xhost_string,*passwd,*res_string;
   int sock_interface,sock_daemons,sockaddr_size,sock_daemon_interface,uid;
   int pipe_structure_host[2],res_read,res_recv,res_recv_host_name,fd;
   dqueue ps_list,ps_list_from_interface;

   xhost_string=(char *)Our_alloc(MAXHOSTNAMELEN+20);
   pid_string=(char *)Our_alloc(PID_SIZE_IN_CHAR);
   message=(char *)Our_alloc(MAX_SIZE);
   uid_string=(char *)Our_alloc(UID_SIZE_IN_CHAR);
   this_host=(char *)Our_alloc(MAXHOSTNAMELEN+1);
   from_host_name=(char *)Our_alloc(MAXHOSTNAMELEN+1);
   host_name=(char *)Our_alloc(MAXHOSTNAMELEN+1);
   structure_remote_host=(struct host_ps *)Our_alloc(sizeof(struct host_ps));
   structure_host_from_daemon=
     (struct host_ps *)Our_alloc(sizeof(struct host_ps));
   command=(char *)Our_alloc(MAX_SIZE);
   passwd=(char *)Our_alloc(16);
   res_string=(char *)Our_alloc(3);

   sockaddr_size=sizeof(ad_sock_interface);

   /* Creation and binding of the socket daemon-interface. */
   sock_interface=Create_sock_UNIX(&ad_sock_interface,OUR_SOCKET_UNIX);
   unlink(ad_sock_interface.sa_data);
   if (bind(sock_interface,&ad_sock_interface,
	    sizeof(ad_sock_interface)) == -1){
     fprintf(stderr,"Error while binding socket daemon-interface.\n");
     exit(-1);
   }
   listen(sock_interface,MAX_REQUESTS);

   /* Creation and binding of the socket daemon-daemons. */
   sock_daemons=Create_sock_INET(&ad_sock_daemons,DAEMONS_PORT);
   if (bind(sock_daemons,&ad_sock_daemons,sizeof(ad_sock_daemons)) == -1){
     fprintf(stderr,"Error while binding socket daemon-daemons.\n");
     exit(-1);
   }
   
   if (pipe(pipe_structure_host) == -1){
     fprintf(stderr,"Error while creating pipe in daemon.\n");
     exit(-1);
   }
     
   fd=open("/dev/null",O_WRONLY);
   ps_list=dq_create();

   /* To avoid defunct processes. */
   signal(SIGCLD,Sigchld);

   if (fork()){
     while(1) {
       sock_daemon_interface=accept(sock_interface,&ad_sock_interface,
				    &sockaddr_size);
       if (fork() == 0){
	 *passwd=0;
	 res_read=Read_nb(sock_daemon_interface,passwd,NULL);
	 if ((!strcmp(passwd,XPROC_PASSWD)) && (res_read == 0)){
	   res_read=Read_nb(sock_daemon_interface,uid_string,NULL);

	   /* Loop while the interface asks informations or sends command to
	      be executed. */
	   while (res_read == 0) {
	     *host_name=0;
	     ps_list_from_interface=Receive_order(sock_daemon_interface,
						  host_name,command);
	     dq_go_beginning(ps_list_from_interface);
	     if ((*host_name == 0) || (!strcmp(host_name,"XBYE")))

	       /* Something is wrong or the interface has gone => the daemon 
		  goes back to the accept */
	       break;
	     if (fork() == 0){
	       if(host_ptr=gethostbyname(host_name)){
		 
		 /* 'host_name' is a recognized host. */
		 Write_nb(sock_daemon_interface,KNOWN_HOST,NULL);
		 bcopy(host_ptr->h_addr,&ad_sock_daemons.sin_addr,
		       host_ptr->h_length);
		 ad_sock_daemons.sin_family=PF_INET;
		 ad_sock_daemons.sin_port=DAEMONS_PORT;
		 if (!strcmp(command,"XGIVE_PS")){
		   Send_message(sock_daemons,uid_string,&ad_sock_daemons);
		   close(pipe_structure_host[1]);
		   *res_string=0;
		   Read_nb(pipe_structure_host[0],res_string,NULL);
		   if (strcmp(res_string,"0"))
		     Write_nb(sock_daemon_interface,TROUBLE,NULL);
		   else {
		     if (Receive_struct_host(Read_nb,
					     pipe_structure_host[0],NULL,
					     structure_remote_host) == -1)
		       Write_nb(sock_daemon_interface,TROUBLE,NULL);
		     else {

		       /* The structure is now filled => it is sent to the 
			  interface. */
		       Write_nb(sock_daemon_interface,NO_TROUBLE,NULL);
		       Send_struct_host(Write_nb,sock_daemon_interface,NULL,
					structure_remote_host);
		     }
		   }
		 }
		 else {
		   Send_message(sock_daemons,command,&ad_sock_daemons);
		   dq_go_beginning(ps_list_from_interface);
		   while(!dq_at_end(ps_list_from_interface)){
		     
		     /* Every pid are sent to the daemon who has to 
			exec 'command' on them. */
		     sprintf(pid_string,"%d",
			     ((struct ps *)
			      dq_get(ps_list_from_interface))->pid);
		     Send_message(sock_daemons,pid_string,&ad_sock_daemons);
		     dq_go_forward(ps_list_from_interface);
		   }
		   Send_message(sock_daemons,"XEND_P",&ad_sock_daemons);
		   dq_go_beginning(ps_list_from_interface);
		   if (dq_empty(ps_list_from_interface)){
		     
		     /* If there is no pid, "xhost + 'host_name'" is needed. To
			make this command successful, 
			"setenv DISPLAY unix:0.0" is a security. */
		     
		     if (fork() == 0){
		       execlp("setenv","setenv","DISPLAY","unix:0.0",NULL);
		       
		       /* execlp may return -1. */
		       exit(-1);
		     }
		     if (fork() == 0){

		       /* We make a dup on /dev/null to hide messages put by
			  xhost. */
		       close(1);
		       dup(fd);
		       execlp("/usr/bin/X11/xhost","/usr/bin/X11/xhost","+",
			      host_name,NULL);
		       exit(-1);
		     }
		     
		     /* We now send the name of this host because the remote
			daemon will have to display the result on this host. */
		     gethostname(this_host,MAXHOSTNAMELEN);
		     Send_message(sock_daemons,this_host,&ad_sock_daemons);
		   }
		   dq_clear(ps_list_from_interface);
		 }
	       }
	       else 

		 /* 'host_name' is not recognized. */
		 Write_nb(sock_daemon_interface,UNKNOWN_HOST,NULL);
	       
	       /* This son must end => exit(0). */
	       exit(0); 
	     }
	   }
	 }
	 
	 /* End of son. */
	 exit(0);
       }
     }
   }
   else
     { 
       while(1){
	 while (Recv_message(sock_daemons,message,&ad_sock_daemons) == -1);
	 if (fork() == 0){
	   if (!strcmp(message,"XSEND_REMOTE_STRUCT")){

	     /* Local daemon */
	     close(pipe_structure_host[0]);
	     if (Receive_struct_host(Recv_message,
				     sock_daemons,
				     &ad_sock_daemons,
				     structure_host_from_daemon) == -1)
	       Write_nb(pipe_structure_host[1],TROUBLE,NULL);
	     else {
	       Write_nb(pipe_structure_host[1],NO_TROUBLE,NULL);

	       /* The structure is filled => it is sent to the father. */
	       Send_struct_host(Write_nb,pipe_structure_host[1],NULL,
				structure_host_from_daemon);
	     }
	   }
	   else {
	     
	     /* Remote daemon */
	     if ((uid=atoi(message)) || 
		 ((message[0] == '0') && (message[1] == 0))) {

	       /* 'message' is a uid (it may be 0 -> root) => we have to
		return the list of ps of the user on the remote host. */
	       Send_message(sock_daemons,"XSEND_REMOTE_STRUCT",
			    &ad_sock_daemons);
	       structure_on_the_other_host=Init_structure_host(uid);

	       /* The structure is filled => it is sent to the daemon who
		  asked it. */
	       Send_struct_host(Send_message,sock_daemons,&ad_sock_daemons,
				structure_on_the_other_host);
	     }
	     else {

	       /* 'message' is a command who has to be executed on the remote
		  host. */
	       res_recv=Recv_message(sock_daemons,pid_string,&ad_sock_daemons);
	       while (strcmp(pid_string,"XEND_P") && (res_recv == 0)){
		 cell_ps=(struct ps*)Our_alloc(sizeof(struct ps));
		 cell_ps->pid=atoi(pid_string);
		 dq_insert(ps_list,cell_ps);
		 res_recv=Recv_message(sock_daemons,pid_string,
				       &ad_sock_daemons);
	       }
	       dq_go_beginning(ps_list);
	       if (res_recv == 0){
		 res_recv_host_name=0;
		 if (dq_empty(ps_list)){

		   /* If no pid are indicated, the result of the command will
		      be displayed on the host which sent the command. */
		   res_recv_host_name=Recv_message(sock_daemons,from_host_name,
						   &ad_sock_daemons);
		   if (res_recv_host_name == 0)
		     sprintf(message,"%s %s:0.0",message,from_host_name);
		 }

		 if (res_recv_host_name == 0)
		   if (fork() == 0){
		     Exec_command(message,ps_list);
		     exit(0);
		   }
	       }
	       dq_clear(ps_list);
	     }
	   }

	   /* End of son. */
	   exit(0);
	 }
	 else

	   /* wait(0) to avoid going back immediately to the Recv_message 
	      function. */
	   wait(0);  

       }
     }
 }



