/* Last modification : 1st March 1992 by the xproc group :
   - malloc replaced by Our_alloc;
   - dq_clear(dqueue) added.
*/

#include <stdio.h>
#include <assert.h>
#include "dqueue.h"

void dq_go_beginning(dqueue q) { q -> cursor = q -> head;}
void dq_go_end(dqueue q) { q -> cursor = q -> tail -> prev;}
int  dq_at_beginning(dqueue q) { return q -> cursor == q -> head;}
int  dq_at_end(dqueue q) { return q -> cursor == q -> tail -> prev;}
int  dq_empty(dqueue q) { return q -> head -> next == q -> tail;}

extern void *Our_alloc(long);

void dq_go_forward(dqueue q) 
{
  assert(!dq_at_end(q));
  q -> cursor = q -> cursor -> next;
}

void dq_go_backward(dqueue q) 
{
  assert(!dq_at_beginning(q));
  q -> cursor = q -> cursor -> prev;
}

void * dq_get(dqueue q) 
{
  assert(!dq_at_end(q));
  return q -> cursor -> next -> element;
}

void * dq_get_first(dqueue q) 
{
  assert(!dq_empty(q));
  return q -> head -> next -> element;
}

void dq_insert(dqueue q, void *element)
{
  dq_cell temp = (dq_cell) Our_alloc(sizeof(struct dq_cell_structure));
  temp -> element = element;
  temp -> next = q -> cursor -> next;
  temp -> prev = q -> cursor;
  q -> cursor -> next -> prev = temp;
  q -> cursor -> next = temp;
  q -> cursor = temp;
}

void dq_delete(dqueue q)
{
  dq_cell temp;  
  assert(!dq_at_end(q));
  temp = q -> cursor -> next;
  temp -> next -> prev = temp -> prev;
  temp -> prev -> next = temp -> next;
  (*(q -> element_destroy))(temp -> element);
  free(temp);
}
  
void
dq_replace(dqueue q, void *element)
{
  dq_cell temp;  
  assert(!dq_at_end(q));
  temp = q -> cursor -> next;
  (*(q -> element_destroy))(temp -> element);
  temp -> element = element;
}  

void dq_destroy(q)
     dqueue q;
{
  dq_go_beginning(q);
  while(!dq_empty(q))
    {
      dq_delete(q);
    }
  free(q->head);
  free(q->tail);
  free(q);
}

void dq_set_destroy(dqueue q,void (*destroy)())
{
  q -> element_destroy = destroy;
}

void dq_push_cursor(dqueue q)
{
  dq_insert(q -> cursor_save, q -> cursor);
  dq_go_beginning(q -> cursor_save);
  q -> cursor = q -> head;
}

void dq_pop_cursor(dqueue q)
{
  q -> cursor = dq_get(q -> cursor_save);
  dq_delete(q -> cursor_save);
}

static void dummy_destroy(void *element)
{
  /* do nothing */
}

static dqueue
dq_create_aux()
{
  dqueue temp;
  temp = (dqueue) Our_alloc(sizeof(struct queue_structure));
  temp -> element_destroy = dummy_destroy;
  temp -> head = (dq_cell) Our_alloc(sizeof(struct dq_cell_structure));
  temp -> tail = (dq_cell) Our_alloc(sizeof(struct dq_cell_structure));
  temp -> cursor = temp -> head;
  temp -> head -> next = temp -> tail;
  temp -> tail -> prev = temp -> head;
  return(temp);
}

dqueue 
dq_copy(dqueue q)
{
  dqueue temp = dq_create();
  dq_push_cursor(q);
  for(dq_go_beginning(q); !dq_at_end(q); dq_go_forward(q))
    {
      dq_insert(temp, dq_get(q));
    }
  dq_pop_cursor(q);
  return temp;
}

dqueue dq_create()
{
  dqueue temp = dq_create_aux();
  temp -> cursor_save = dq_create_aux();
  return temp;
}

void dq_clear(dqueue q)
{
  dq_go_beginning(q);
  while(!dq_at_end(q))
    dq_delete(q);
}
