/*****************************************************************************/
/** File    : execute.c                                                     **/
/** Purpose : Sending and receiving order from the interface                **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/param.h>
#include <sys/user.h>
#include "structure.h"
#include "defines.h"
#include "dqueue.h"


extern int Read_nb(int,
		   char *,
		   struct sockaddr_in *);    /* From manip_sock.c */
extern void Write_nb(int,
		     char *,
		     struct sockaddr_in *);  /* From manip_sock.c */
extern void *Our_alloc(long);                /* From our_alloc.c */


int Execute(int sock,
	    char *host_name,
	    char *command,
	    dqueue ps_list)
{
  /* Send command to be executed on 'host_name' to daemon. 
     Used by the interface.
     Return value : 0 if successful, -1 otherwise. */

  char *host_state,*pid;
  int res_read;

  pid=(char *)Our_alloc(PID_SIZE_IN_CHAR);
  host_state=(char *)Our_alloc(3);
  Write_nb(sock,host_name,NULL);
  Write_nb(sock,command,NULL);
  dq_go_beginning(ps_list);
  while (!dq_at_end(ps_list)){
    sprintf(pid,"%d",((struct ps *)dq_get(ps_list))->pid);

    /* Each pid is sent. */
    Write_nb(sock,pid,NULL);
    dq_go_forward(ps_list);
  }
  Write_nb(sock,"XEND_P",NULL);
  res_read=Read_nb(sock,host_state,NULL);

  /* 'host_state' contains "0" if 'host_name' id recognized, "-1" otherwise. */
  return ((res_read == -1)?-1:atoi(host_state));
}

dqueue Receive_order(int sock,
		     char *host_name,
		     char *command)
{
  /* Receive command to be executed on 'host_name'.
     Used by the daemon.
     Return value : the list of process on which command has to be executed. */

  struct ps *cell_ps;
  char *ps_pid;
  int res_read;
  long nb=0;
  dqueue ps_list;

  ps_pid=(char *)Our_alloc(PID_SIZE_IN_CHAR);

  ps_list=dq_create();

  /* No timeout here. */
  res_read=read(sock,(char *)&nb,sizeof(long));
  read(sock,host_name,nb);
  if ((res_read == 0) || (!strcmp(host_name,"XBYE")))

    /* Something is wrong or the interface has gone. */
    return ps_list;

  Read_nb(sock,command,NULL);
  Read_nb(sock,ps_pid,NULL);
  while (strcmp(ps_pid,"XEND_P")){
    cell_ps=(struct ps *)Our_alloc(sizeof(struct ps));
    cell_ps->pid=atoi(ps_pid);
    dq_insert(ps_list,cell_ps);
    Read_nb(sock,ps_pid,NULL);
  }
  dq_go_beginning(ps_list);
  return ps_list;
}
      

    
    
