/*****************************************************************************/
/** File    : init_resour.c                                                 **/
/** Purpose : create application, load and parse database resource file     **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include "xproc.h"
#include "interface.h"

#include "fallback.h"

extern void initialize_pixmap ();
extern void parse_cfile ();

extern void UnState (Widget, XButtonEvent*); /* state.c */
extern void State (Widget, XButtonEvent*); /* state.c */
extern void Close_Host (Widget , XButtonEvent*); /* actions.c */
extern void * Our_XtMalloc(long); /* main.c */

XtActionsRec actions[] = {
  {"State",State},
  {"Close_Host",Close_Host},
  {"UnState",UnState},
  {NULL, NULL}
};

void Initialize (int arc, String **arv)
{
  /* initialize application and actions
     read and parse config file 
     read and parse File Database */
     
  XrmDatabase db_source;  
  String db_file;

  /* add a new argument -cf  */
  XtResource resources[] = {
    "configFile", "ConfigFile", XtRString, sizeof (String),
    XtOffset (AppDataPtr, cffile), XtRImmediate, (caddr_t) NULL,
  };
  XrmOptionDescRec options[] = {
    {"-cf", ".configFile", XrmoptionSepArg, NULL}
  };

  if ((xproc_home = (String)getenv("HOME")) == NULL)
    {
      fprintf(stderr, "Warning: can't get environment variable HOME\n");
      xproc_home = XtNewString("");
    } else
      xproc_home = XtNewString(xproc_home);

  toplevel = XtAppInitialize (&app_con,
			      "Xproc", 
			      options,
			      XtNumber(options),
			      &arc,arv, 
			      fallback_ressources,
			      NULL, ZERO);

  /* if argument -cf file found app_data.cffile=file */
  XtGetApplicationResources (toplevel,
			     &app_data,
			     resources,
			     XtNumber(resources),
			     NULL,ZERO);

  /* add action Close_Host, State and UnState to context */
  XtAppAddActions (app_con,actions, XtNumber(actions));  

  parse_cfile (); /* parse the config file */

  /* read database file and parse it */
  db_xproc = XtDatabase (XtDisplay(toplevel));

  db_file=(String) Our_XtMalloc((strlen(xproc_home)+6)*sizeof(char));
  strcpy(db_file,xproc_home);
  strcat(db_file,"/Xproc");
    
  db_source = XrmGetFileDatabase (db_file);
  if(db_source==NULL)
    {
      fprintf(stderr,
	      "Warning : Local DataBase File Xproc not found \n");
      XtFree(db_file);
    }  
  
  if(db_source!=NULL)
    XrmMergeDatabases (db_source, &db_xproc);
  
  Initialize_pixmap (); /* initialise default pixmap */
}




