/*****************************************************************************/
/** File    : list.c                                                        **/
/** Purpose : most handling list functions                                  **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include "xproc.h"

#include <X11/Shell.h>
  
#include "Xedw/XedwList.h"
  
#include "structure.h"
#include "interface.h"
  
extern Pixmap StringtoPixmap (String ,Pixmap); /* modif_icon.c */
extern struct host_ps * Goto_host (String ); /* goto.c */
extern struct icon_name * Goto_struct (dqueue,String );  /* goto.c */
extern int Give_ps (int,String ,struct host_ps *); 
extern void Problem_w (String); /* problem.c */

extern Pixmap command_pixmap;
extern Pixmap host_pixmap;
extern Pixmap ps_pixmap;
extern int sock;
  
XedwList ** Create_command_list(void)
{
  /* build list of command for XedwList widget */

  XedwList ** list;
  int i = 0;
  
  list= (XedwList**)Our_XtMalloc (sizeof(XedwList*)*
			       Nb_struct(config_list[Command]));
  
  dq_go_beginning(config_list[Command]);
  
  while (!dq_at_end(config_list[Command]))
    {
      list[i]= (XedwList*)Our_XtMalloc (sizeof(XedwList));
      /* icon of command */
      list[i]->icon=StringtoPixmap 
	(((struct icon_name *)dq_get(config_list[Command]))->icon,
	 command_pixmap);
      /* name of command */
      list[i]->string=XtNewString 
	(((struct icon_name *)dq_get(config_list[Command]))->name);
      i++;
      dq_go_forward(config_list[Command]);
    }
  return list;
}

XedwList ** Create_host_list (void)
{
  /* build list of host for XedwList widget */
  struct icon_name * tmp_net_host;
  XedwList **list;
  int i = 0;
  
  list= (XedwList**)Our_XtMalloc (sizeof(XedwList*)*Nb_struct(config_list[Host]));
  
  dq_go_beginning(config_list[Host]);
  
  while (!dq_at_end(config_list[Host]))
    {
      list[i]= (XedwList*)Our_XtMalloc (sizeof(XedwList));
      /* icon of host */
      list[i]->icon=StringtoPixmap 
	(((struct icon_name *)dq_get(config_list[Host]))->icon,
	 host_pixmap);
      /* name of host */
      list[i]->string=XtNewString
	(((struct icon_name *)dq_get(config_list[Host]))->name);
      i++;
      dq_go_forward(config_list[Host]);
    }
  return list;
}

XedwList ** Create_host_ps_list (String  name_host)
{
  /* build list of processes for XedwList widget */
  XedwList **list;
  dqueue tmp_ps;
  struct icon_name * pt_icon_name;
  String  name_ps;
  struct ps * current_ps;
  int i = 0;
  
  tmp_ps = (Goto_host (name_host))->list_ps;
  current_ps=(struct ps *)Our_XtMalloc(sizeof(struct ps));

  list  = (XedwList**)Our_XtMalloc (sizeof(XedwList*)*(Nb_ps_by_host(name_host)+1));
  list[0]=NULL;
  dq_go_beginning(tmp_ps);
  while (!dq_at_end(tmp_ps))
    {
      list[i]=(XedwList*)Our_XtMalloc (sizeof(XedwList));
      tmp_ps = (Goto_host (name_host))->list_ps;
      current_ps=(struct ps *)dq_get(tmp_ps);
      
      /* has current_ps an icon ? */
      pt_icon_name = Goto_struct (config_list[Ps],
				  current_ps->name);
      if (pt_icon_name==NULL)    
	list[i]->icon = ps_pixmap;
      else
	list[i]->icon = StringtoPixmap (pt_icon_name->icon,
					ps_pixmap);
      name_ps = (String)Our_XtMalloc
	((strlen(current_ps->name)+10)*sizeof(char));
      sprintf (name_ps,"%s(%d)",
	       current_ps->name,
	       current_ps->pid);
      /* ps_name(ps_pid) as name of item */
      list[i]->string = name_ps;
      i++;
      dq_go_forward(tmp_ps);
    }
  return list;
}

void Update_Highlighted_ps_List (void)
{
  int i;
  for (i=0; i<nb_w_ps ; i++)
    w_ps[i].list.highlighted_ps = XedwListShowCurrent (w_ps[i].list.itself);
}

int highlightedPsList (String * list_built)
{
  /* build list of all processes highlighted on all host
     "Process(pid) on host" in list_built and 
     return the total number of these processes */
     
  String * tmp;
  XedwListReturnStruct *tmp_list;
  Arg arglist[1];
  String  name;
  int i,counter=0;
  
  tmp=list_built;
  
  for(i=0;i<nb_w_ps;i++)
    {
      tmp_list = w_ps[i].list.highlighted_ps;
      
      XtSetArg (arglist[0], XtNtitle,&name);
      XtGetValues (w_ps[i].toplevel, arglist, 1);	  
      
      if(tmp_list-> xedwList_index!=-1)
	while (tmp_list!=NULL)
	  {
	    tmp[counter] = (String)Our_XtMalloc(sizeof(char)*
						(strlen(tmp_list->string)+
						 strlen(" on ")+strlen(name)));
	    sprintf (tmp[counter],"%s on %s",tmp_list->string,name);
	    tmp_list =tmp_list->next;
	    counter++;
	  }
    }
  return counter;
}

void Unhighlight_all (Widget w)
{
  /* unhighlighted all icons in widget w */
  XedwListReturnStruct *list;
  
  list = XedwListShowCurrent (w);
  while (list!=NULL && list->xedwList_index!=-1)
    {
      XedwListUnhighlight (w,list->xedwList_index);
      list = list->next;
    }
}

int Nb_struct (dqueue dq)
{
  int nb=0;
  
  dq_go_beginning(dq);
  while (!dq_at_end(dq))
    {
      nb++;  
      dq_go_forward(dq);
    }
  return nb;
}

int Nb_ps_by_host (String  name_host)
{

  dqueue list_ps;
  int nb = 0;
  
  list_ps = Goto_host(name_host)->list_ps;
  dq_go_beginning(list_ps);
  while (!dq_at_end(list_ps))
    {
      dq_go_forward(list_ps);
      nb++;
    }
  return nb;
}

void Create_host_ps_struct (void)
{
  /* create dqueue host_ps_list with
     all host which are in config_list[Host] */
  
  struct host_ps * tmp_host_ps;

  host_ps_list = dq_create();

  dq_go_beginning(host_ps_list);
  dq_go_beginning(config_list[Host]);

  while (!dq_at_end(config_list[Host]))
    {
      tmp_host_ps = (struct host_ps *)Our_XtMalloc (sizeof(struct host_ps));
      tmp_host_ps->list_ps = dq_create();
      tmp_host_ps->name    = XtNewString(((struct host_ps *)
					  dq_get(config_list[Host]))->name);
      dq_insert(host_ps_list, tmp_host_ps);
      dq_go_forward(config_list[Host]);
    }
}      

int Ask_daemon_and_build_list(String host_name , Widget widget_list)
{
  struct host_ps * tmp_host_ps;
  
  tmp_host_ps=(struct host_ps *)Our_XtMalloc(sizeof(struct host_ps));
  if(Give_ps(sock,host_name,tmp_host_ps)==-1)
    {
      Problem_w(host_name);
      return -1;
    }
  else
    {
      (Goto_host(host_name))->list_ps=tmp_host_ps->list_ps;
      XedwListChange(widget_list,
		     Create_host_ps_list(host_name),
		     Nb_ps_by_host(host_name),
		     0,True);
    }
  return 0;
}





