/*****************************************************************************/
/** File    : main_interfa.c                                                **/
/** Purpose : create main interface window ( commands + hosts )             **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/
#include "xproc.h"

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
  
#include "interface.h"
#include "structure.h"
  
extern XedwList ** Create_command_list (void); /* list.c */
extern XedwList ** Create_host_list (void); /* list.c */
extern int Nb_struct (dqueue); /* list.c */
extern void Action_received (Widget,XtPointer,XtPointer); /* actions.c */
extern void Quit (Widget,XtPointer,XtPointer); /* actions.c */ 
extern void Update (Widget,XtPointer,XtPointer); /* actions.c */

void Create_Main_Interface (void)
{
  /* create main interface in 3 parts:
     1) Widget Box with 2 buttons : Update and Quit
     2) Widget Command : list of commands
     3) Widget Host : list of hosts
     mouse button actions must be specified 
     in fallback.h or/and Xproc database */

  Arg arglist[2];

  w_main.itself = 
    XtCreateManagedWidget ("Main",
			   formWidgetClass,
			   toplevel,
			   NULL,ZERO);
  
  w_main.box.itself = 
    XtCreateManagedWidget ("Box",
			   boxWidgetClass,
			   w_main.itself,
			   NULL,ZERO);
  
  w_main.box.info = 
    XtCreateManagedWidget ("UpDate",
			   commandWidgetClass,
			   w_main.box.itself,
			   NULL,ZERO);
  
  w_main.box.quit = 
    XtCreateManagedWidget ("Quit",
			   commandWidgetClass,
			   w_main.box.itself,
			   NULL,ZERO);
  
  w_main.command.itself = 
    XtCreateManagedWidget ("Command",
			   formWidgetClass,
			   w_main.itself,
			   NULL,ZERO);
  
/* Widget Command must be under Widget Box */
  XtSetArg (arglist[0], XtNfromVert, w_main.box.itself);
  XtSetValues (w_main.command.itself,arglist,1);
  
  w_main.host.itself = 
    XtCreateManagedWidget ("Host",
			   formWidgetClass,
			   w_main.itself,
			   NULL,ZERO);
  /* Widget Host must be under Widget Box and 
     at the right of Widget Command */
  XtSetArg (arglist[0], XtNfromHoriz, w_main.command.itself);
  XtSetArg (arglist[1], XtNfromVert, w_main.box.itself);
  XtSetValues (w_main.host.itself,arglist,2);
  
  /* install scrollbars on list of commands */
  w_main.command.scroll = 
    XtCreateManagedWidget ("Scroll",
			   viewportWidgetClass,
			   w_main.command.itself,
			   NULL,ZERO);
  /* install scrollbars on list of hosts */
  w_main.host.scroll = 
    XtCreateManagedWidget ("Scroll",
			   viewportWidgetClass,
			   w_main.host.itself,
			   NULL,ZERO);
  
  w_main.command.list.itself = 
    XtCreateManagedWidget ("List_command",
			   xedwListWidgetClass,
			   w_main.command.scroll,
			   NULL,ZERO);

  w_main.host.list.itself = 
    XtCreateManagedWidget ("List_host",
			   xedwListWidgetClass,
			   w_main.host.scroll,
			   NULL,ZERO);
  /* list of command and hosts are build */
  XedwListChange (w_main.command.list.itself,
		  Create_command_list (),
		  Nb_struct (config_list[Command]),
		  0,True);

  XedwListChange (w_main.host.list.itself,
		  Create_host_list(),
		  Nb_struct(config_list[Host]),
		  0,True);

  /* add callbacks :
     Action_received (Command) on Widget Command
     Action_received (Host) on Widget Host
     Update () and Quit() on Widget Box */

  XtAddCallback (w_main.command.list.itself,XtNcallback,
		 Action_received,Command);
  XtAddCallback (w_main.host.list.itself,XtNcallback,
		 Action_received,Host);
  XtAddCallback (w_main.box.quit,XtNcallback,
		 Quit, NULL);
  XtAddCallback (w_main.box.info,XtNcallback,
		 Update, NULL);
  
  XtRealizeWidget (toplevel);
}

