/*****************************************************************************/
/** File    : manip_host.c                                                  **/
/** Purpose : sending and receiving structure of processes                  **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/param.h>
#include <sys/user.h>
#include "structure.h"
#include "defines.h"
#include "dqueue.h"


extern void *Our_alloc(long);   /* From our_alloc.c */


void Send_struct_host(int (*send_func)(),
		      int sock_dest,
		      struct sockaddr_in *ad_sock,
		      struct host_ps * structure_host)
{
  /* Send a structure of processes using send_func (Write_nb or Send_message). 
     Used by the daemon. 
     Return value : none. */

  struct ps *cell_ps;
  dqueue temp_ps;
  char *pid,*ppid,*state;

  pid=(char *)Our_alloc(PID_SIZE_IN_CHAR);
  ppid=(char *)Our_alloc(PID_SIZE_IN_CHAR);
  state=(char *)Our_alloc(3);
  cell_ps=(struct ps *)Our_alloc(sizeof(struct ps));
  
  send_func(sock_dest,structure_host->name,ad_sock);
  temp_ps=structure_host->list_ps;
  dq_go_beginning(temp_ps);
  while(!dq_at_end(temp_ps)){
    cell_ps=(struct ps *)dq_get(temp_ps);

    /* Each field of cell_ps is sent. */
    send_func(sock_dest,cell_ps->name,ad_sock);
    while (*cell_ps->arg)
      send_func(sock_dest,*(cell_ps->arg++),ad_sock);
    send_func(sock_dest,"XEND_ARG",ad_sock);
    sprintf(pid,"%d",cell_ps->pid);
    send_func(sock_dest,pid,ad_sock);
    sprintf(ppid,"%d",cell_ps->ppid);
    send_func(sock_dest,ppid,ad_sock);
    sprintf(state,"%d",cell_ps->state);
    send_func(sock_dest,state,ad_sock);
    dq_go_forward(temp_ps);
  }
  send_func(sock_dest,"XEND_STRUCT",ad_sock);
}


int Receive_struct_host(int (*recv_func)(),
			int sock_from,
			struct sockaddr_in *ad_sock,
			struct host_ps *structure_host)
{
  /* Receive a structure of processes using recv_func (Read_nb or 
     Receive_message).
     Used by the daemon. 
     Return value : 0 if successful, -1 otherwise. */

  struct ps *cell_ps;
  dqueue ps_list;
  char *ps_name,*ps_pid,*ps_ppid,*ps_state,*ps_arg;
  int i,res_select,current_res;
  fd_set fdset;
  
  ps_arg=(char *)Our_alloc(MAX_SIZE);
  ps_name=(char *)Our_alloc(MAXCOMLEN+1);
  ps_pid=(char *)Our_alloc(PID_SIZE_IN_CHAR);
  ps_ppid=(char *)Our_alloc(PID_SIZE_IN_CHAR);
  ps_state=(char *)Our_alloc(2);
  structure_host->name=(char *)Our_alloc(MAXHOSTNAMELEN);
  
  ps_list=dq_create();
  FD_ZERO(&fdset);
  FD_SET(sock_from,&fdset);
  *ps_name=0;

  if ((res_select=recv_func(sock_from,structure_host->name,ad_sock)) != -1)
    res_select=recv_func(sock_from,ps_name,ad_sock);

  while ((res_select != -1) && strcmp(ps_name,"XEND_STRUCT")){
    
    /* Each field is cought */
    cell_ps=(struct ps *)Our_alloc(sizeof(struct ps));
    cell_ps->arg=(char **)Our_alloc(MAX_SIZE);
    cell_ps->name=(char *)Our_alloc(MAXCOMLEN+1);	  
    
    strcpy(cell_ps->name,ps_name);
    res_select=recv_func(sock_from,ps_arg,ad_sock);
    i=0;
    while((res_select != -1) && strcmp(ps_arg,"XEND_ARG")){
      cell_ps->arg[i]=(char *)Our_alloc(MAXCOMLEN+1);
      strcpy(cell_ps->arg[i++],ps_arg);
      res_select=recv_func(sock_from,ps_arg,ad_sock);
    }
    cell_ps->arg[i]=NULL;

    if (res_select != -1){
      res_select=recv_func(sock_from,ps_pid,ad_sock);
      cell_ps->pid=atoi(ps_pid);
    }
    else 
      break;
    if (res_select != -1){
      res_select=recv_func(sock_from,ps_ppid,ad_sock);
      cell_ps->ppid=atoi(ps_ppid);
    }
    else 
      break;
    if (res_select != -1){
      res_select=recv_func(sock_from,ps_state,ad_sock);
      cell_ps->state=atoi(ps_state);
    }
    else 
      break;
    dq_insert(ps_list,cell_ps);
    if (res_select != -1)
      res_select=recv_func(sock_from,ps_name,ad_sock);
  }
  dq_go_beginning(ps_list);
  structure_host->list_ps=ps_list;
  return res_select;
}

