/*****************************************************************************/
/** File    : manip_sock.c                                                  **/
/** Purpose : operations on sockets                                         **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include "defines.h"


int Read_nb(int sock,
	    char *message,
	    struct sockaddr_in *ad_sock)
{

  /* Read a message in sock with timeout (local communication). Each message 
     is preceded by his length. 
     Used by the daemon. 
     Return value : 0 if successful, -1 otherwise (if timeout occured). */

  struct timeval time_value;
  int descriptor_table_size;
  long nb;
  fd_set fdset;

  /* Data for timeout. */
  time_value.tv_sec=TIMEOUT;
  time_value.tv_usec=0L;
  descriptor_table_size=getdtablesize();

  FD_ZERO(&fdset);
  FD_SET(sock,&fdset);
  if (select(descriptor_table_size,&fdset,0,0,&time_value))
    read(sock,(char *)&nb,sizeof(long));
  else 
    return -1;
  if (select(descriptor_table_size,&fdset,0,0,&time_value))
    read(sock,message,nb);
  else 
    return -1;
  return 0;
}

void Write_nb(int sock,
	      char *message,
	      struct sockaddr_in *ad_sock)
{
  /* Write the length of message then the message itself in sock (local 
     communication). 
     Used by the daemon. 
     Return value : none. */

  long nb;
  
  nb=strlen(message)+1;
  write(sock,(char *)&nb,sizeof(long));
  write(sock,message,nb);
}

void Send_message(int sock,
		  char *message,
		  struct sockaddr_in *ad_sock)
{
  /* Send a message in sock (network communication). 
     Used by the daemon. 
     Return value : none. */

  sendto(sock,message,strlen(message)+1,DGRAM_OPTION,ad_sock,
	 sizeof(struct sockaddr_in));
}

int Recv_message(int sock,
		 char *message,
		 struct sockaddr_in *ad_sock)
{
  /* Receive a message in sock with timeout (network communication). 
     Used by the daemon. 
     Return value : 0 if successful, -1 otherwise (if timeout occured). */

  struct timeval time_value;
  int descriptor_table_size;
  int sockaddr_size;
  fd_set fdset;

  /* Data for timeout. */
  time_value.tv_sec=TIMEOUT;
  time_value.tv_usec=0L;
  descriptor_table_size=getdtablesize();

  sockaddr_size=sizeof(struct sockaddr_in);
  FD_ZERO(&fdset);
  FD_SET(sock,&fdset);
  if (select(descriptor_table_size,&fdset,0,0,&time_value)){
    recvfrom(sock,message,MAX_SIZE,DGRAM_OPTION,ad_sock,&sockaddr_size);
    return 0;
  }
  return -1;
}


int Create_sock_UNIX(struct sockaddr *ad_sock,
		     char *name)
{
  /* Create a socket named 'name' for local communication. 
     Used by the daemon and the interface. 
     Return value : the descriptor of the socket. If a problem occured, 
     we leave the program. */

  int sock;

  sock=socket(PF_UNIX,SOCK_STREAM,PROTOCOL);
  if (sock == -1){
    fprintf(stderr,"Error while creating socket daemon-interface.\n");
    exit(-1);
  }
  ad_sock->sa_family=PF_UNIX;
  strcpy(ad_sock->sa_data,name);
  return sock;
}

int Create_sock_INET(struct sockaddr_in *ad_sock,
		     int port)
{
  /* Create a socket for network communication (using 'port'). 
     Used by the daemon. 
     Return value : the descriptor of the socket. If a problem occured, 
     we leave the program. */

  int sock;

  sock=socket(PF_INET,SOCK_DGRAM,PROTOCOL);
  if (sock == -1){
    fprintf(stderr,"Error while creating socket daemon-daemons.\n");
    exit(1);
  }
  ad_sock->sin_family=PF_INET;
  ad_sock->sin_port=port;
  ad_sock->sin_addr.s_addr=INADDR_ANY;
  strcpy(ad_sock->sin_zero,"00000000");
  return sock;
}

