/*****************************************************************************/
/** File    : modif_icon.c                                                  **/
/** Purpose : handling of pixmap in a  XedwList                             **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include "xproc.h"

#include "Xedw/XedwList.h"

#include "interface.h"
#include "structure.h"

extern struct icon_name * Goto_struct (dqueue,String ); /* goto.c */
extern void * Our_XtMalloc(long); /* main.c */

extern GC xprocgc; 
  
XedwList ** Modiflist_icon (Widget w, String  icon_name, 
			     Pixmap src, Pixmap dst, int function)
{
  /* copy or mask the icon of the item named 
     icon_name in XedwList w */

  XedwList ** list,** tmp_list;
  Pixmap newpixmap;
  Arg arglist[1];
  
  XtSetArg (arglist[0], XtNxedwList, &list);
  XtGetValues (w, arglist, 1);
  
  tmp_list=list;
  while ((strcmp((*tmp_list)->string, icon_name)) && 
	 (tmp_list!=NULL))
    tmp_list++;
  if (tmp_list==NULL) return NULL; /* item icon_name not found in XedwList */
  
  switch (function){
  case GXand: /* mask icon_name pixmap in XedwList by pixmap dst */
    newpixmap = XCreatePixmap (XtDisplay(w),
			       RootWindowOfScreen(XtScreen(w)),
			       32, 32,
			       DefaultDepthOfScreen(XtScreen(w)));
    XSetFunction (XtDisplay (w), xprocgc, GXcopy);
    XCopyPlane (XtDisplay(w),
		(*tmp_list)->icon, newpixmap,
		xprocgc, 0, 0, 32, 32, 0, 0, 1);
    XSetFunction (XtDisplay(w), xprocgc, GXand);
    XCopyPlane (XtDisplay(w),
		dst, newpixmap,
		xprocgc, 0, 0, 32, 32, 0, 0, 1);
    
    (*tmp_list)->icon = newpixmap;
    break;
  case GXcopy: /* copy */
    (*tmp_list)->icon = src; 
    break;
  }
  return list;
}

Pixmap StringtoPixmap (String  name_file,Pixmap pixmap_default)
{
  /* read bitmap file name_file 
     if not exist return pixmap_default 
     else return new pixmap */

  Pixmap newpixmap;
  unsigned int *width, *height;
  int * x,*y;
  int status;
  
  newpixmap = XCreatePixmap (XtDisplay(toplevel),
			     RootWindowOfScreen(XtScreen(toplevel)),
			     32, 32,
			     DefaultDepthOfScreen(XtScreen(toplevel)));
  
  width  = (unsigned int*)Our_XtMalloc (sizeof(unsigned int));
  height = (unsigned int*)Our_XtMalloc (sizeof(unsigned int));
  x      = (int*)Our_XtMalloc (sizeof(int));
  y      = (int*)Our_XtMalloc (sizeof(int));
  
  status = XReadBitmapFile (XtDisplay(toplevel),
			    RootWindowOfScreen(XtScreen(toplevel)),
			    name_file,
			    width,height,
			    &newpixmap,
			    x,y);
 
  XtFree(width);XtFree(height);
  XtFree(x);XtFree(y);
 
  if (status!=0)
    return pixmap_default;

  return newpixmap;
}



