%{
/*****************************************************************************/
/** File    : parser.y                                                      **/
/** Purpose : Parser generator used to recognize grammatical rules .        **/
/**           Its function is to analyse the configuration file .           **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include <ctype.h>
#include "structure.h"
#include "dqueue.h"

int key,err=0;
void assignment();
int boolean();

%}

%start prog

%union
{
char *chaine;
}

%token <chaine> DEFINE STRING NAME ICON FUNCT COMMAND HOST PS NODISPLAY
%type <chaine> prog definition struct_list structure end_structure string
               key_word 

%%

prog		: prog definition 
		| definition
		;

definition	: DEFINE key_word '{' struct_list '}'
                | error { yyerror("<define> not found");}
		;

struct_list	: struct_list '{' structure end_structure
                | {}
                | error '}' { yyerror("< { > not found");}
		;

structure       :  NAME  string
                       { assignment(key,$2,"",$2,1); }
                |  NAME string 
                   ICON string 
                       { assignment(key,$2,$4,$2,1); }
                |  NAME  string 
                   ICON  string 
     		   FUNCT string 
                       { assignment(key,$2,$4,$6,1); }
                |  NAME  string 
                   FUNCT string 
                   NODISPLAY string
                       { assignment(key,$2,"",$4,boolean($6)); }
                |  NAME  string 
                   FUNCT string 
                       { assignment(key,$2,"",$4,1); }
                |  NAME  string 
                   ICON  string 
     		   FUNCT string 
                   NODISPLAY string
                       { assignment(key,$2,$4,$6,boolean($8)); }
                | error{ yyerror("NAME is a mandatory field in a definition");}
                ;

end_structure   : '}'  {}
                | error  { yyerror("< } > not found");}
                ;

string		: STRING   { $$=(char *)malloc(strlen($1)-1);
 			     $1++;
			     $1[strlen($1)-1]='\0';
			     strcpy($$,$1);}
		;

key_word	: COMMAND { key=COMMAND; }
                | HOST    { key=HOST; }
                | PS      { key=PS; }
                | error   { yyerror("Type definition not found");}
		;

%%
#include "lexical.c"

void assignment(int key,
                char *name,
                char *icon,
                char *function,
		int parameters)
{ 
  /* An object of struct icon_name'type is created and
  its fields are assigned with different values return by LEX */

  struct icon_name *cell;
  cell=(struct icon_name*)malloc(sizeof(struct icon_name));
  cell->name=(char *)malloc(sizeof(char)*strlen(name));
  strcpy(cell->name,name);
  cell->icon=(char *)malloc(sizeof(char)*strlen(icon));
  strcpy(cell->icon,icon);
  cell->function=(char *)malloc(sizeof(char)*strlen(function));
  strcpy(cell->function,function);
  cell->parameters=parameters;

  /* This object is insert in one of the three lists depending
  on the key : (item return by LEX)
         key = COMMAND   ->  list[0]
         key = HOST      ->  list[1]
         key = PS        ->  list[2] */

  dq_insert(config_list[key - COMMAND],cell);  
}

int boolean(char *rep)
{
  /* Function used to convert a string'answer in boolean (integer) */

  if(!(strcmp(rep,"NO")&
       strcmp(rep,"N")& 
       strcmp(rep,"no")))
    return 0;
  else
    return 1;
}


int yywrap(void)
{
  /* Function called when EOF encountered */
  return 1;
}

  
yyerror(char *s)
{
  /* Function called when YACC find errors */

  err=1;
  if(strcmp(s,"syntax error"))
	fprintf(stderr,"File .xprocrc -line %d : %s\n",yylineno,s);
}


void parser(void)
{
  /* Function called to active analysers */

  int st,i;
  for(i=Command;i<Ps+1;i++)
    config_list[i]=dq_create();
  yyparse();
  if(err) exit(-1);
}  


